
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                                   BB_estim_dens
%
%
%	Procedure BB_estim_dens for estimating the spectral density of a process having stationary increments
%                          (example: FBM, OU,...) observed at random time
%
%  Input: * traj: path of the process
%         * t: random time of the observation of the process (in s)
%         * fmin: minimum of the frequencies for estimating the spectral density (in Hz)
%         * fmax: maximum of the frequencies for estimating the spectral density (in Hz)
%         * printout: 1 for printing a graph
%
%  Output:* ff: estimation of the spectral density
%         * frequence: frequencies of estimation
%         * ffm and ffM: confidence interval of the estimation
%         * H: the local fractality parameter on the frequency domain (only one linear zone is observed)
%         * HH1,HH2: both the local fractality parameters on the frequency domain (two linear zones are observed)
%
%
%  Example          [ff,frequence,ffm,ffM,H,HH1,HH2]=BB_estim_dens(traj,t,0.1,10,20,1)
%                          
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [ff,frequence,ffm,ffM,H,HH1,HH2]=BB_estim_dens_coeur(traj,t,fmin,fmax,M,printout)

if nargin<5, 
   printout=0;
end
   
if size(traj'*traj)>1
   traj=traj';
end
   
if size(t'*t)>1
   t=t';
end


p=0;
coeff1=0;
T=0;
psidb=0;
H=0;
n=0;
xx=0;
INN=0;
NN=0;
Htilde=0;
ftest=0;
temps=0;
nbpoints=0;
L=0;

r=0.1;
rho=0.5;

%[p,psidb,xx]=wavefun('meyr',15);
load Meyer4  


ff=0;
SS=0;


nbpoints=length(traj)
L=t(2:nbpoints)-t(1:(nbpoints-1));
moy=mean(L);
%t=cumsum(L);
delta=(nbpoints)^(-0.5)/moy; % Pas d'echantillonnage
temps=delta*t;
Tn=temps(nbpoints);
mn=0.1*Tn;
Mn=0.9*Tn;
taun=Mn-mn;
MM=M-1;   

%------------------ Parametres ----------------------

q=fmax/fmin; % Saut entre les echelles
frequencevraie=fmin*q.^((0:MM)/MM); 
%frequencevraie=fmin+((0:MM)/MM)*(fmax-fmin);  % Frequences choisies en unite du temps
frequence=frequencevraie/delta;
dd=log(15)/log(1000+nbpoints); % Taille fenetre
Nf=floor(log(nbpoints)^3); % Nombre de points N_n


%--------------------Calculs des estimateurs-----------------

Ma=0;
SN=0;
b=mn+taun*(0:(Nf-1))/(Nf-1); % Shifts  choisis
    t1=temps(1:(nbpoints-1));
    t2=temps(2:nbpoints);
for n=1:length(frequence)
    frequence(n);
    Ma=0;
    SS=0;
    tic  
       Ma=0;
       SS=0;
    for j=1:Nf
        coeff1=0;
        %coeff1=(temps(2:(nbpoints))-temps(1:(nbpoints-1))).*(phi2(frequence(n)*(temps(1:(nbpoints-1))-b(j)),psidb,xx)+phi2(frequence(n)*(temps(2:(nbpoints))-b(j)),psidb,xx))/2;
	     %coeff1=(t2-t1).*(exp(i*frequence(n)*(t1-b(j))).*phi2((frequence(n)*(t1-b(j))*nbpoints^(-dd)),psidb,xx)+exp(i*frequence(n)*(t2-b(j))).*phi2((frequence(n)*(t2-b(j))*nbpoints^(-dd)),psidb,xx))/2;
	     coeff1=(t2-t1).*exp(i*frequence(n)*(t1-b(j))).*phi2((frequence(n)*(t1-b(j))*nbpoints^(-dd)),psidb,xx);        
        Ma(j)=abs(sum(coeff1'.*traj(1:(nbpoints-1))'))^2; % Calcul du module au carr du coefficient d'ondelette pour le shift b(j)
    end
    SS=frequence(n)*mean(Ma)/(2*pi*35*nbpoints^dd)%*delta^(1+2*H)
    %SS=frequence(n)*mean(Ma) % Variance empirique des coefficients d'ondelette pour la frequence 
    ff(n)=SS;
    sigma(n)=2*sqrt(pi)*ff(n)/sqrt(frequence(n))*sqrt(0.1142);
    ffm(n)=max(0,ff(n)-1.96*sigma(n)/(nbpoints^(0.25-dd/2))); % Intervalle de confiance
    ffM(n)=ff(n)+1.96*sigma(n)/(nbpoints^(0.25-dd/2)); % Intervalle de confiance
    toc
    save BB_biomec_fbmy1 ff ffm ffM nbpoints frequence frequencevraie delta dd
 end
 
 N=log(frequencevraie);
 IN=log(ff);
 [H,K]=densH1(N,IN);

 
 for p=3:(M-3)
    N1=0;
    IN1=0;
    N1=log(frequencevraie(1:p));
    IN1=log(ff(1:p));
    [H1(p),K1(p)]=densH1(N1,IN1);
    N2=0;
    IN2=0;
    N2=log(frequencevraie((p+1):M));
    IN2=log(ff((p+1):M));
    [H2(p),K2(p)]=densH1(N2,IN2);
    Q(p)=sum((IN1+(2*H1(p)+1)*N1-K1(p)).^2)+sum((IN2+(2*H2(p)+1)*N2-K2(p)).^2);
 end
 popt=find(Q==min(Q(3:M-3)))
 frequencevraie(popt)
 NN1=log(frequencevraie(1:popt));
 NN2=log(frequencevraie((1+popt):M));
 HH1=H1(popt)
 KK1=K1(popt);
 HH2=H2(popt)
 KK2=K2(popt);

 

if printout==1
   clf        
   
   figure(1)
hold
plot(frequencevraie,ff,':')
plot(frequencevraie,ffm,'r')
plot(frequencevraie,ffM,'r')
xlabel('Chosen frequencies (Hz) ')
ylabel('Estimator of f ')
hold off

figure(2)
hold
plot(log(frequencevraie),log(ff),':')
plot(log(frequencevraie),log(ffm),'r')
plot(log(frequencevraie),log(ffM),'r')
plot(N,-(2*H+1)*N+K,'k')
xlabel('Logarithm of the chosen frequencies (Hz) ')
ylabel('Logarithm of the estimator of f ')
hold off
       
figure(3)
clf
hold
plot(log(frequencevraie),log(ff),':')
plot(log(frequencevraie),log(ffm),'r')
plot(log(frequencevraie),log(ffM),'r')
plot(NN1,-(2*HH1+1)*NN1+KK1,'k')
plot(NN2,-(2*HH2+1)*NN2+KK2,'k')
xlabel('Logarithm of the chosen frequencies (Hz) ')
ylabel('Logarithm of the estimator of f ')
hold off

end

